;Monday, May 04, 2020, 11:35:25 AM
;for use with atmega8
;
;handles 90S1200's better
;this code also programs both avr & 89S51/89S52
;you must select AT89S8252 in avrprog
;dropped device support for AT89S53, AT89S8252
;
;program fuses for atmega8 using 7.3728 MHz crystal
;low:  0xFF
;high: 0xD9
;
;edit delay.inc if you are using a different crystal than 7.3728 MHz
;edit usart.inc if you are using a different crystal than 7.3728 MHz
;***************************************************************************
.nolist
.include "m8def.inc"
.list
;
;***************************************************************************
;*
;* CONSTANTS
;*
;***************************************************************************
.equ PGESZE  = 0x10		; default Page size for Programming
.equ BUFSIZE = 0x40		; 64 bytes internal RAM for Buffer
					; change this to speed up programming
					; avrprog may not work well if this value is changed
;***************************************************************************
;*
;* PORTS
;*	Ports Definitions
;*
;* DESCRIPTION
;*	Change the following definitions if the RESET pin to the
;*	target moves and/or if the SCK/MISO/MISO/LED moves.
;*
;***************************************************************************
;PORTB
.equ	SPI_PORT = PORTB
.equ	SPI_PIN  = PINB
.equ	SPI_DDR  = DDRB
;
.equ	SCK		= 0		; SCK pin of the target (output)
.equ	RST		= 1		; RESET pin of the target (output)
.equ	MOSI	= 3 	; MOSI pin of the target (output)
.equ	MISO	= 4		; MISO pin of the target (input)
;
;PORTC
.equ	LED_PORT = PORTC
.equ	LED_DDR  = DDRC
;
;PORTC
.equ	LEDG		= 2	;green LED
.equ	LEDY		= 3	;yellow LED
;
.macro	table				; load Z pointer
	ldi	ZL,low(@0*2)		; low
	ldi	ZH,high(@0*2)		; high
.endm
;***************************************************************************
;*
;* Global Register Variables
;*
;***************************************************************************
;
;r0 used with lpm instruction
.def	cmd1		= r1	; Universal commands params
.def	cmd2		= r2	; ..
.def	cmd3		= r3	; ..
.def	pol_al	= r4	; Polling address low
.def	pol_ah	= r5	; Polling address high
.def	Bcnt1		= r6	; Block Mode Counter1
.def	Bcnt2		= r7	; Block Mode Counter2
.def	Bcnt3		= r8	; Block Mode Counter2
.def  B_Flag	= r9	; Flag for ws_del Routine
.def  B_Mode	= r10	; Marks Block Mode commands (See Note 16)
.def 	Memtype	= r11	; Flag for Memorytype for Block Commands (See Note 16)
.def	Pagewords	= r12	; Size of Controller page to program in Page Mode (Words!) 
.def 	PollcodeF	= r13	; Code for Polling Flash
.def	count		= r14
;
.def	temp1		= r16	; Temporary register 1
.def	temp2		= r17	; Temporary register 2
.def	temp3		= r18	; Temporary register 3
.def	s_data	= r19	; SPI data
.def	u_data	= r20	; UART data
.def	device	= r21	; Device code
.def	lockbyte	= r22	; Param to set correct lockbit read/write mode 
.def	rd_s_data	= r23	; Read data on SPI
.def	pol_cmd	= r24	; Polling command
.def	p_data	= r25	; Polling data
;r26,r27 used as X register
.def	addrl		= r28	; (YL) Low order byte of address
.def	addrh		= r29	; (YH) High order byte of address
;r30,r31 used as Z register
;
;***************************************************************************
.CSEG
.org 0
		rjmp	INIT			; Reset Vector Address
;***************************************************************************
;*
;* INIT
;*
;* DESCRIPTION
;*	Initialization
;*
;***************************************************************************
INIT:
	ldi	temp1,low(RAMEND)		;Initialize stackpointer
	out	SPL,temp1
	ldi	temp1,high(RAMEND)
	out	SPH,temp1
	
	ldi	temp1,PGESZE		; default Pagesize
	mov Pagewords,temp1
	mov Bcnt3,Pagewords		; set counter for Pagesize
	ldi	device,0x20			; S2313 as default
	clr B_Mode				; Flag for Block Modes (see Note 16)
	clr B_Flag				; Flag for eNhanced Block write
	rcall	init_ports 		; Initialize ports
	rcall	release_ports	; Release ports
	rcall	USART_Init		; Initialize USART
	rcall healthcheck			; show that Prog is working after Powerup (LED test)
	 	
;***************************************************************************
;*
;* PROGRAM
;*	waitcmd -> main
;*
;* DESCRIPTION
;*	Wait for and execute commands.
;*
;***************************************************************************
waitcmd:
	rcall	LED_off			;ALL LED's off

waitcmd1:
	rcall	getc			; while (getc() == ESC) {};
	cpi	u_data,0x1b
	breq	waitcmd1

;====== 'T' - Device Type ==================================================
	cpi	u_data,'T'			; 'T' Device type
	brne	w0
	rcall	getc
	mov	device,u_data		; get device type
	table Dev_M				; prepare to set Pagesize
	rcall set_pagesize		; If device has Page Mode support, set Pagesize
	ldi	temp1,0xFF
	mov	PollcodeF,temp1		; preset PollcodeF with 0xFF, will be overwritten if Pollcode known
	table Dev_S				; prepare to set Polling Code
	rcall set_pollcode		; If device has no Page Size support, set code for Polling Flash
	rjmp	put_ret
;====== 'S' - Return Software Identifier ===================================
w0:	
	cpi	u_data,'S'		; 'S' Return software identifier
	brne	w1
	table	ID_Str
	rcall	put_string		; put string "AVR ISP"
	rjmp	waitcmd
;====== 'V' - Return Software Version ======================================
w1:	
	cpi	u_data,'V'		; 'V' Return software version
	brne	w2
	table	SW_Ver
	rcall	put_string		; put software version
	rjmp	waitcmd
;====== 'v' - Return Hardware Version ======================================
w2:	
	cpi	u_data,'v'		; 'v' Return hardware version
	brne	w3
	table	HW_Ver
	rcall	put_string		; put hardware version
	rjmp	waitcmd
;====== 't' - Show Supported Devices =======================================
w3:	
	cpi	u_data,'t'		; 't' Show supported devices
	brne	w4
	table	Dev_S
	rcall	put_table		; put supported devices codes
	table	Dev_M
	rcall	put_table		; put supported devices codes
	ldi	u_data,0x00		; putc(0x00) - end of device list
	rcall	putc
	rjmp	waitcmd
;====== 'p' - Return Programmer Type =======================================
w4:	
	cpi	u_data,'p'		; 'p' Return programmer type
	brne	w5
	ldi	u_data,'S'		; putc('S') - serial programmer
	rcall	putc
	rjmp	waitcmd
;====== 'a' - Return autoincrement address support =========================
w5:	
	cpi	u_data,'a'		; 'a' Return address auto increment
	brne	w51
	ldi	u_data,'Y'		; putc('Y') - supports autoinc
	rcall	putc
	rjmp	waitcmd
;====== 'M' - Return enhanced Mode support (Note 14) ========================
w51:	
;	cpi	u_data,'M'		; 'M' Return enhanced mode Support
;	brne	w52
;	ldi u_data,'Y'		; putc('Y') - supports enhanced Mode
;	rcall putc
;	rjmp	waitcmd
;====== 'i' - Return Chip ID (Note 14) ======================================
w52:
	cpi	u_data,'i'		; 'i' Return Chip ID
	brne	w53
	table	ChipID
	rcall	put_string		; put Chip ID string 
	ldi u_data,0x0a		; putc(LF)
	rcall putc
	rjmp	put_ret
;====== 'b' - Return Block write Mode support (Note 17) ========================
w53:	
	cpi	u_data,'b'			; 'b' Return enhanced mode Support
	brne	w6

	ldi u_data,'Y'			; putc('Y') - supports enhanced Mode
	rcall putc
	ldi	u_data,high(BUFSIZE)	; putc((BUFSIZE>>8) & 0xff);
	rcall putc
	ldi	u_data,low(BUFSIZE)	; putc(BUFSIZE&0xff);
	rcall putc
	rjmp	waitcmd
;====== 'x' - Set LED ======================================================
w6:	
	cpi	u_data,'x'		; 'x' Set LED (LED off or green)
	brne	w61
	rcall	getc			; get parameter
	rcall	LED_Green			;turn on green led
	rjmp	put_ret
;====== 'y' - Clear LED ====================================================
w61:	
	cpi	u_data,'y'		; 'y' Clear LED (LED on or red)
	brne	w7
	rcall	getc			; get parameter
	rcall	LED_Yellow			;turn on yellow led
	rjmp	put_ret
;===========================================================================
; We require that the device code be selected before any of the other commands
w7:
	table	Dev_S			; load pointer
	rcall	bel_table
	brcc	w71			; device belongs to table
	table	Dev_M	
	rcall	bel_table
	brcc	w71			; device belongs to table
	rjmp	put_err		; not match, goto put_err()
;====== 'P' - Enter Programming Mode =======================================
w71:	
	cpi	u_data,'P'		; 'P' Enter programming mode
	breq	w70
	rjmp	w8
w70:	
	rcall	LED_Yellow		;turn on yellow led
	rcall	catch_ports		; enable SPI Pins
	cbi	SPI_PORT,SCK	; clear SCK		
	ldi	temp3,32		; count = 32
	mov	count,temp3
	rcall	pas_RESET		; set RESET passive (high or disabled)
	ldi	temp1,50		; delay 50mS
	rcall	delay
	rcall	act_RESET		; set RESET active (low or enabled)
	ldi	temp1,50		; delay 50mS
	rcall	delay	

	tst device
	brmi	InitS89
w72:
	ldi	s_data,0xac	
	rcall	wrser			; wrser(0xac) SPI write (byte 1)
	ldi	s_data,0x53	
	rcall	wrser			; wrser(0x53) SPI write (byte 2)
					; SPI Synchronization (fix!)
	
s1:	rcall	rdser			; SPI read  (byte 3)
	cpi	device,0x13		; shitty 90S1200 does not echo 53 byte!
	breq	s3

	cpi	s_data,0x53		; slave should return 0x53 byte
	breq	s3			; break

	ldi	s_data,0x00					
	rcall	wrser			; wrser(0x00) SPI write (byte 4)
	rcall	pulse_SCK		; pulse SCK

	ldi	s_data,0xac		
	rcall	wrser			; wrser(0xac) SPI write (byte 1)

	ldi	s_data,0x53		
	rcall	wrser			; wrser(0x53) SPI write (byte 2)

	dec	count			; count-1
	brne	s1			; loop
	rjmp	s3			; else

s2:	ldi	s_data,0x00	
	rcall	wrser			; wrser(0x00) SPI write (byte 3)

s3:	ldi	s_data,0x00	
	rcall	wrser			; wrser(0x00) SPI write (byte 4)

s4:	ldi	temp1,4		; delay 4mS
	rcall	delay
	rjmp	put_ret

InitS89:				;for 89S51/52
	ldi	s_data,0xac	
	rcall	wrser			; wrser(0xac) SPI write (byte 1)
	ldi	s_data,0x53	
	rcall	wrser			; wrser(0x53) SPI write (byte 2)
	ldi	s_data,0x00					
	rcall	wrser			; wrser(0x00) SPI write (byte 3)
	rcall	rdser			; SPI read  (byte 4)

	cpi	s_data,0x69		; if (rdser == 0x69)
	breq	Init2			; break

	rcall	pulse_SCK		; pulse SCK

	dec	count			; count-1
	brne	InitS89		; loop until count = 0

	cpi	s_data,0x69		; if (rdser == 0x69)
	brne	Init3

Init2:	
	rjmp	s4

Init3:
	rjmp	put_err
;====== 'c' - Write Program Memory, Low Byte ===============================
w8:	
	cpi	u_data,'c'		; 'c' Write program memory, low byte
	brne	w9
	rcall	getc			; get data byte

w8b: 	ldi	s_data,0x40
	mov	pol_cmd,s_data	; save command for polling	
	rcall	wrser			; wrser(0x40)   SPI write (byte 1)

	mov	s_data,addrh
	rcall	wrser			; wrser(addrh)  SPI write (byte 2)

	mov	s_data,addrl	
	rcall	wrser			; wrser(addrl)  SPI write (byte 3)

	mov	s_data,u_data	
	rcall	wrser			; wrser(u_data) SPI write (byte 4)

	mov	p_data,u_data	; save data for polling
	mov	pol_al,addrl	; save address for polling
	mov	pol_ah,addrh
	tst	device
	brpl	w83 
	adiw	addrl,1		; Auto increment address for S89 device
w83:
	tst device
	brmi	w84
	rjmp	wait_S		; write FLASH delay
w84:
	rjmp	s4
;====== 'C' - Write Program Memory, High Byte ==============================
w9:	
	cpi	u_data,'C'		; 'C' Write program memory, high byte
	brne	w92
	rcall	getc			; get data byte

w9a:	tst device
	brmi	w91			; S89 device

	ldi	s_data,0x48	
	mov	pol_cmd,s_data	; save command for polling	
	rcall	wrser			; wrser(0x48)   SPI write (byte 1)
	mov	s_data,addrh	
	rcall	wrser			; wrser(addrh)  SPI write (byte 2)
	mov	s_data,addrl	
	rcall	wrser			; wrser(addrl)  SPI write (byte 3)
	mov	s_data,u_data	; wrser(u_data) SPI write (byte 4)
	rcall	wrser
	mov	p_data,u_data	; save data for polling
	mov	pol_al,addrl	; save address for polling
	mov	pol_ah,addrh
	adiw	addrl,1		; Auto increment address

	rjmp	wait_S		; write FLASH delay
w91:
	rjmp	put_err		; S89 device have byte wide program memory!
;====== 'B' Block Write Memory (see Note 17) ======================
w92:	
	cpi	u_data,'B'		; 'B' Block Write Program Memory
	breq w92a
	rjmp w10
w92a:
	rcall	getc			; get count High Byte
	tst u_data
	breq w92b
	rjmp put_err
;ldi u_data,1	
;rjmp	put_debug

w92b:
	rcall	getc			; get count Low Byte
	cpi u_data,BUFSIZE+1	; check maximum count 
	brlo w92c
	rjmp put_err
;ldi u_data,2
;rjmp	put_debug	

w92c:
	mov Bcnt1, u_data		; ignore BUFSIZE high Byte (must be 0 here)
	mov Bcnt2, u_data

	rcall	getc			; get Memory type to write
	mov Memtype,u_data	; Flag for Memtype

	ldi XH,HIGH(SRAM_START)
	ldi XL, LOW(SRAM_START)	; set X pointer to SRAM begin

w93:
	rcall	getc			; get data until Bcnt1 is reached	
	st X+, u_data		; store data to SRAM
	dec Bcnt1
	brne w93

	ldi XH,HIGH(SRAM_START)
	ldi XL, LOW(SRAM_START)	; set X pointer to SRAM begin

	ldi temp3,1
	mov B_Mode,temp3		; B_Mode != 0

	mov u_data,Memtype	; restore Memtype 
	cpi	u_data,'F'		; 'F' Flash Memory ist to write
	breq	w94a

	cpi	u_data,'E'		; 'E' eeprom Memory ist to write
	breq	w94			; Entry Point for Data Memory Block write

	rjmp put_err
;ldi u_data,3		; not 'F' or 'E', send error message
;rjmp	put_debug

w94:					; write Data (EEPROM) Memory
	ld u_data,X+
	rcall eeprom_write

	dec Bcnt2
	brne w94
	rjmp put_ret

;Entry Point for Flash Block write
w94a:
	tst device
	brmi	w95d			; S89 device

;sts	BufCntH,ZH		;save Z
;sts	BufCntL,ZL

	table	Dev_M	
	rcall	bel_table
	brcc w95			; device belongs to table if carry clr
	rjmp	w96			; not match, goto "non Paged" Mode

w95:					; Entry Point for AVRProg Block Flash write
;lds	ZH,BufCntH		;restore Z
;lds	ZL,BufCntL

	ldi	s_data,0x40		; write low byte
	rcall spi123

	ld	s_data, X+
	rcall	wrser			; wrser(s_data) SPI write (byte 4)

	ldi	s_data,0x48		; write high byte
	mov	pol_cmd,s_data	; save command for polling	
	rcall spi123

	ld	s_data, X+
	mov	p_data,s_data	; save data for polling
	mov	pol_al,addrl	; save address for polling
	mov	pol_ah,addrh
	rcall	wrser			; wrser(u_data) SPI write (byte 4)

	adiw	addrl,1		; Auto increment address

	dec Bcnt2
	breq w95b
	dec Bcnt2
	breq w95b
	tst B_Mode
	breq w95			; Loop for "Block Mode Commands" see Note 16
	dec Bcnt3			; count Bytes for Pagesize
	brne w95
w95b:
	tst B_Mode
	brne w95c
	rjmp put_ret
w95d:
	rjmp	w97
w95c:
	dec Bcnt3			; Bcnt3 will be set correct after write

	ldi	s_data,0x4c		; write Memory Page
	rcall	wrser			; wrser(0x4c)  SPI write (byte 1)

	mov	s_data,pol_ah	; last written Address is in Page to be programmed!
	rcall	wrser			; wrser(addrh) SPI write (byte 2)

	mov	s_data,pol_al	; last written Adress is in Page to be programmed!
	rcall	wrser			; wrser(addrl) SPI write (byte 3)

	ldi	s_data,0x00
	rcall	wrser			; wrser(0x00)  SPI write (byte 4)

	mov Bcnt3,Pagewords	; reload Counter for Pagesize
	rjmp wait_M

w96:	
				; Non Page Mode Flash write
	ld	p_data, X+		; load byte to Poll data
	cpi	p_data,0xFF		; if (p_data == 0xFF) skip burning
	breq	w961			; 

	ldi	s_data,0x40		; write low byte
	mov	pol_cmd,s_data	; save command for polling	
	rcall spi123

	mov	s_data,p_data	; reload data from Poll data
	rcall	wrser			; wrser(s_data) SPI write (byte 4)

	mov	pol_al,addrl	; save address for polling
	mov	pol_ah,addrh
	rcall Bws_pol

w961:
	dec Bcnt2
	ld	p_data, X+		; load byte to Poll data
	cpi	p_data,0xFF		; if (p_data == 0xFF) 
	breq	w962

	ldi	s_data,0x48		; write high byte
	mov	pol_cmd,s_data	; save command for polling	
	rcall spi123
	mov	s_data,p_data	; reload data from Poll data
	rcall	wrser			; wrser(u_data) SPI write (byte 4)

w96a:	mov	pol_al,addrl	; save address for polling
	mov	pol_ah,addrh
	rcall Bws_pol
w962:
	adiw	addrl,1		; Auto increment address
	dec Bcnt2
	brne w96
	rjmp put_ret		; reply

w96b:					
	ldi	s_data,0x40
	mov	pol_cmd,s_data	; save command for polling	
	rcall	wrser			; wrser(addrh)  SPI write (byte 1)

	mov	s_data,addrh	; s_data = (addrh << 3) | 0x02;
	rcall	wrser			; wrser(addrh)  SPI write (byte 2)

	mov	s_data,addrl	
	rcall	wrser			; wrser(addrl)  SPI write (byte 3)

	ld	s_data, X+	
	mov	p_data,s_data	; save data for polling
	rcall	wrser			; wrser(u_data) SPI write (byte 4)
	rjmp	w96a

w97:
	ldi	s_data,0x40	
	rcall	wrser			; wrser(addrh)  SPI write (byte 1)

	mov	s_data,addrh	; s_data = (addrh << 3) | 0x02;
	rcall	wrser			; wrser(addrh)  SPI write (byte 2)

	mov	s_data,addrl	
	rcall	wrser			; wrser(addrl)  SPI write (byte 3)

	ld	s_data, X+	
	rcall	wrser			; wrser(u_data) SPI write (byte 4)

	adiw	addrl,1		; Auto increment address

	ldi	temp1,1		; delay 1mS;
	rcall	delay

	dec Bcnt2
	brne w97			; do all bytes in ram
	rjmp	put_ret
;====== 'R' - Read Program Memory ==========================================

w10:	
	cpi	u_data,'R'		; 'R' Read program memory
	brne	w10B

	tst device
	brmi	rpm			; S89 device

	ldi	s_data,0x28		; read high Byte (order is different from Block Read!)
	rcall	wrser			; wrser(0x28) SPI write (byte 1)

	mov	s_data,addrh
	rcall	wrser			; wrser(addrh) SPI write (byte 2)

	mov	s_data,addrl	
	rcall	wrser			; wrser(addrl) SPI write (byte 3)

	rcall	rdser			; SPI read  (byte 4)
	mov	u_data,s_data
	rcall	putc			; send data

rpm:
	ldi	s_data,0x20		; read Low Byte
	rcall	wrser			; wrser(0x20)  SPI write (byte 1)

	mov	s_data,addrh	
	rcall	wrser			; wrser(addrh) SPI write (byte 2)

	mov	s_data,addrl	
	rcall	wrser			; wrser(addrl) SPI write (byte 3)

	rcall	rdser			; SPI read  (byte 4)
	mov	u_data,s_data
	rcall	putc			; send data

rpm3:
	adiw	addrl,1		; Auto increment address
	rjmp	waitcmd		; goto waitcmd()
;====== 'g' - Block Read Memory (See Note 17) ======================
w10B:	
	cpi	u_data,'g'		; 'g' Block Read Program Memory
	brne	w11

	rcall	getc			; XH = length high
	mov	XH,u_data

	rcall	getc			; XL = length low
	mov	XL,u_data

	rcall	getc			; Memorytype (F = flash, E = eeprom)
	cpi u_data,'F'
	breq w10B2

	cpi u_data,'E'
	breq w10B1

	rjmp put_err
w10B1:
	rcall read_send_datamem
	sbiw XL, 1
   	brne w10B1
	rjmp waitcmd			; goto waitcmd

w10B2:
 	rcall read_send_progmem
	tst device
	brmi w10B3
	sbiw XL, 2
w102B2a:
	brne w10B2
	rjmp waitcmd
w10B3:
	sbiw XL, 1				;do all bytes in X counter
	rjmp	w102B2a
;
;   	brne w10B2
;	rjmp waitcmd			; goto waitcmd
;====== 'A' - Load Address =================================================
w11:	
	cpi	u_data,'A'		; 'A' Load address
	brne	w12
	rcall	getc			; addrh = getc();
	mov	addrh,u_data
	rcall	getc			; addrl = getc();
	mov	addrl,u_data
	rjmp	put_ret		; goto reply();

;====== 'D' - Write Data Memory ============================================
w12:	
	cpi	u_data,'D'		; 'D' Write data memory
	brne	w13
	rcall	getc			; get data
	rcall eeprom_write
	rjmp	put_ret
;====== 'd' - Read Data Memory =============================================
w13:	
	cpi	u_data,'d'		; 'd' Read data memory
	brne	w14
	rcall   read_send_datamem
	rjmp	waitcmd		; goto waitcmd
;====== 'L' - Leave Programming Mode =======================================
w14:	
	cpi	u_data,'L'		; 'L' Leave programming mode
	brne	w142
w141:
	rcall	pas_RESET	; set RESET passive
	rcall	release_ports	; release ports
	rcall	LED_Green		; turn on green led
	rjmp    put_ret
;====== 'E' - Exit , release all Ports, inhibit AVR910 =====================
w142:	
	cpi	u_data,'E'		; 'E' xit
	brne	w15
	rjmp w141			; exit command for AVR Prog	
;====== 'e' - Chip Erase ===================================================
w15:	
	cpi	u_data,'e' 		; 'e' Chip erase
	brne	w16

	ldi	s_data,0xac
	rcall	wrser			; wrser(0xac) SPI write (byte 1)
	
	ldi	s_data,0x80
	rcall	wrser			; wrser(0x80) SPI write (byte 2)
w151:
	ldi	s_data,0xFF
	rcall	wrser			; wrser(0x00) SPI write (byte 3)

	ldi	s_data,0xFF
	rcall	wrser			; wrser(0x00) SPI write (byte 4);

	ldi	temp1,0x20		; delay 20mS
	rcall	delay

	sbi SPI_PORT,RST

	ldi	temp1,0xFF		; delay 250mS
	rcall	delay
	ldi	temp1,0xFF		; delay 250mS
	rcall	delay
	rcall	release_ports
	rjmp	w70			;enable programming mode again
;	rjmp	put_ret
;====== 'l' - Write Lockbits ==============================================

; writes Lockbits in SPI Byte 2 for old "classic" AVR Chips, and in addition
; writes Lockbits in SPI byte 4 for new Mega and Tiny Devices. 
; One Type does not care about the other Byte...
; also does the S51 lockbits PROPERLY!

; Some more different Addresses for other devices?

w16:	
	cpi	u_data,'l'		; 'l' Write Lockbits
	brne	w161
	rcall	getc			; get data

	tst	device
	brmi	w16a			; S89 device

	ldi	s_data,0xac
	rcall	wrser			; wrser(0xac)   SPI write (byte 1)
	
	mov	s_data,u_data 
	ori	s_data,0xe0
	rcall	wrser			; wrser(u_data) SPI write (byte 2)
	rjmp 	w162

w16a:					;for 89S51/89S52 devices
	ldi	temp1,0x04		; do it four times
	mov	cmd3,temp1

	ldi	temp1,0xE0
	mov	cmd2,temp1
Lock1:
	ldi	s_data,0xac
	rcall	wrser			; wrser(0xac)   SPI write (byte 1)
	
	mov	s_data,cmd2
	rcall	wrser			; wrser(u_data) SPI write (byte 2)

	ldi	s_data,0xFF
	rcall	rdser			; rdser(0xFF)   SPI write (byte 3)
	mov 	lockbyte,s_data	; save returned byte
	
	ldi	s_data,0xFF
	rcall	wrser			; wrser(0xFF)   SPI write (byte 4)
	
	inc 	cmd2
	ldi	temp1,50		; delay 50mS
	rcall	delay
	dec	cmd3
	brne	Lock1

	cpi	lockbyte,0xE3
	brne	Lock2

	ldi	temp1,10		; delay 10mS
	rcall	delay
	rjmp	put_ret

Lock2:
	rjmp	put_err
;====== 'f' - Write Fusebits ==============================================
w161:
	cpi	u_data,'f'		; 'f' Write Fusebits
	brne	w17
	rcall	getc			; get data

	ldi	s_data,0xac
	rcall	wrser			; wrser(0xac)   SPI write (byte 1)

	ldi	s_data,0xa0
	rcall	wrser			; wrser(0xa0)   SPI write (byte 2)
w162:
	clr s_data
	rcall	wrser			; wrser(0x00)   SPI write (byte 3)

	mov	s_data,u_data 
	rcall	wrser			; wrser(u_data) SPI write (byte 4)
w164:
	ldi	temp1,10		; delay 10mS
	rcall	delay
	rjmp	put_ret
;====== 's' - Read Signature Bytes =========================================
w17:	
	cpi	u_data,'s'		; 's' Read signature bytes
	brne	w18
	tst	device		; S89 device
	brmi	w171a	

	ldi 	temp3,0x30		;first spi byte = 0x30
	mov 	cmd1,temp3		;save to cmd1
	clr 	cmd2			;cmd2 = 0x00
	ldi 	temp3,0x02		;cmd3 = 0x02
	mov 	cmd3,temp3

	rcall w123r4		;write spi cmd1,cmd2,cmd3,getc byte
	dec 	cmd3			;
	rcall	w123r4
	clr 	cmd3
	rcall w123r4
w17a:
	rjmp 	waitcmd

w171a:
	ldi	temp3,0x02
	mov	cmd3,temp3
	rcall	w17call

	dec	cmd3
	rcall	w17call

	dec	cmd3
	rcall	w17call
	rjmp	w17a

w17call:
	ldi	s_data,0x28
	rcall	wrser

	mov	s_data,cmd3
	rcall	wrser

	ldi	s_data,0x00
	rcall	wrser

	rcall	rdser
	mov	u_data,s_data
	rcall	putc
	ret
;====== 'm' - Write Program Memory Page ====================================
w18:	
	cpi	u_data,'m'		; 'm' Write Program Memory Page
	brne	w19

	ldi	s_data,0x4c	
	rcall	wrser			; wrser(0x4c)  SPI write (byte 1)

;	mov	s_data,addrh	; original, reload address
	mov	s_data,pol_ah	; for speeding up transfer
	rcall	wrser			; wrser(addrh) SPI write (byte 2)

;	mov	s_data,addrl	; original, reload address
	mov	s_data,pol_al	; for speeding up transfer
	rcall	wrser			; wrser(addrl) SPI write (byte 3)

	ldi	s_data,0x00
	rcall	wrser			; wrser(0x00)  SPI write (byte 4)
	rjmp	wait_M		; write FLASH delay
;====== ':' - Universal Command ============================================
w19:	
	cpi	u_data,':'		; ':' Universal Command
	brne	w20
	rcall	getc			; get data1
	mov	cmd1,u_data		; cmd1 = data1
	rcall	getc			; get data2
	mov	cmd2,u_data		; cmd2 = data2
	rcall	getc			; get data3
	mov	cmd3,u_data		; cmd3 = data3
	rcall w123r4
	ldi	temp1,50		; delay 50mS
	rcall	delay
	rjmp	put_ret
;====== '.' - New Universal Command ========================================
w20:	
	cpi	u_data,'.'		; '.' New Universal Command
	brne	w21
	rcall	getc			; get data1
	mov	cmd1,u_data		; cmd1 = data1
	rcall	getc			; get data2
	mov	cmd2,u_data		; cmd2 = data2
	rcall	getc			; get data3
	mov	cmd3,u_data		; cmd3 = data3
	rcall	getc			; get data4
	rcall w1234
	mov	u_data,rd_s_data
	rcall	putc			; send data
	ldi	temp1,50		; delay 50mS
	rcall	delay
	rjmp	put_ret	
;====== 'r' - Read Lock Bits ==============================================
w21:	
	cpi	u_data,'r'		; 'r' Read Lockbits
	brne	w22
	ldi	s_data,0x58		; SPI load (byte 1)
	rjmp	w22a
;====== 'F' - Read Fusebits (lfuse) ========================================
w22:
	cpi	u_data,'F'		; 'F' Read Fusebits
	brne	w23
	ldi	s_data,0x50		; SPI load (byte 1)
w22a:
	rcall	wrser			; wrser(0x5n)   SPI write (byte 1)
	clr s_data			; SPI load (byte 2)
	rjmp w26
;====== 'N' - Read high Fusebits ==========================================
w23:
	cpi	u_data,'N'		; 'N' Read high Fusebits
	brne	w24
	ldi	s_data,0x58
	rcall	wrser			; wrser(0x58)   SPI write (byte 1)
	rjmp	w25

;====== 'Q' - Read extended Fusebits ========================================
w24:
	cpi	u_data,'Q'		; 'Q' Read extended Fusebits
	brne	w30
	ldi	s_data,0x50		; SPI load (byte 1)
	rcall	wrser			; wrser(0x50)   SPI write (byte 1)
w25:
	ldi	s_data,0x08		; SPI load (byte 2)
w26:
	rcall	wrser			; wrser(0x00)   SPI write (byte 2)
	clr s_data
	rcall	wrser			; wrser(0x00)   SPI write (byte 3)
	rcall	rdser			;               SPI read  (byte 4)
	mov	u_data,s_data
	rcall	putc			; send data
	rjmp waitcmd
;====== Unknown Command ====================================================
w30:	rjmp	put_err
;====== Wait for FLASH write in avr910 mode ==================================
wait_S:
	table	Dev_M	
	rcall	bel_table
	brcs	ws_pol		; No Page Mode, poll last Byte
	rjmp	put_ret		; in Byte Mode time is long enougt to get next Byte...
Bws_pol:				; entry address for Block mode delay
	ldi temp3,1
	mov B_Flag,temp3		; set Flag that ws_poll comes back with ret ...

ws_pol:
	tst	PollcodeF		; if polling not applicable, standart delay
	breq	ws_del		; polling not used
ws_def:
	cp 	p_data,PollcodeF	; if (data == PollcodeF)
	breq 	ws_del		; wait default delay
	andi	pol_cmd,0x0f	; write command: 0x48, 0x40
	ori	pol_cmd,0x20	; read  command: 0x28, 0x20
	clr	temp3			; clear polling counter
ws_cy:
	mov	s_data,pol_cmd	
	rcall	wrser			; wrser(pol_cmd) SPI write (byte 1)
	mov	s_data,pol_ah
	rcall	wrser			; wrser(pol_ah) SPI write (byte 2) (S89=byte1)

	mov	s_data,pol_al	
	rcall	wrser			; wrser(pol_al) SPI write (byte 3) (S89=byte2)

	rcall	rdser			;               SPI read  (byte 4) (S89=byte3)				
	cp	s_data,p_data
	breq	ws_ok			; s_data = p_data
	dec	temp3
	brne	ws_cy			; loop
ws_del:				; 256 polling cycles are over, give additional standard Time
	ldi	temp1,10		; delay 10mS
	rcall	delay
ws_ok:
	tst B_Flag
	breq put_ret
	clr B_Flag			; Reset B_Flag for normal operation 
	ret
;====== Wait for FLASH write in page mode ==================================
wait_M:
	cpi	device,0x41
	breq	wm_del		; polling inapplicable for m103
	cpi	device,0x23
	breq	poll_t2313		; polling different for t2313
	cpi	p_data, 0xFF	; if last Byte was 0xFF give standart delay
	breq	wm_del
	andi	pol_cmd,0x0f	; write command: 0x48, 0x40
	ori	pol_cmd,0x20	; read  command: 0x28, 0x20
	clr	temp3			; clear polling counter
wm_cy:
	mov	s_data,pol_cmd	
	rcall	wrser			; wrser(pol_cmd) SPI write (byte 1)
	mov	s_data,pol_ah
	rcall	wrser			; wrser(pol_ah) SPI write (byte 2)
	mov	s_data,pol_al	
	rcall	wrser			; wrser(pol_al) SPI write (byte 3)
	rcall	rdser			;               SPI read  (byte 4)
	cp	s_data,p_data
	breq	wm_ok			; s_data = p_data
	dec	temp3
	breq	wm_del		; 256 polling cycles are over, give another 50ms delay...
	rjmp	wm_cy			; loop
wm_del:
	ldi	temp1,50		; delay 50mS
	rcall	delay
wm_ok:
	tst B_Mode
	breq wm_end

	tst Bcnt2
	breq wm_end
	rjmp w95
wm_end:
	clr B_Mode			; Reset Block Mode Flag
	rjmp put_ret

poll_t2313:				; Polling ATTiny2313 is different
	ldi	pol_cmd,0xf0	; Byte1= 0xF0
	clr	temp3			; clear polling counter
	mov	pol_ah,temp3	; Byte2= 0x00
p2313_1:
	mov	s_data,pol_cmd	
	rcall	wrser			; wrser(pol_cmd) SPI write (byte 1)
	mov	s_data,pol_ah
	rcall	wrser			; wrser(pol_ah) SPI write (byte 2)
	mov	s_data,pol_al	
	rcall	wrser			; wrser(pol_al) SPI write (byte 3)
	rcall	rdser			;               SPI read  (byte 4)
	andi	s_data,0x01		; last Bit tells if polling OK
	breq	wm_ok			; s_data = p_data
	dec	temp3
	breq	wm_del		; 256 polling cycles are over, give another 50ms delay...
	rjmp	p2313_1		; loop
;====== Command Error ======================================================
put_debug:				; shows u_data on serial for debug reason				
	rcall putc
put_err:
	ldi	u_data,'?'		; putc('?')
	rcall	putc			; send '?'
	rjmp	waitcmd		
;====== Reply Command ======================================================
put_ret:
	ldi	u_data,0x0d		; putc(0x0D)
	rcall	putc			; send CR
	rjmp	waitcmd
;***************************************************************************
;*
;* FUNCTION
;*	set_pagesize
;*
;* DESCRIPTION
;*	sets programming Page size for selected Device.
;*
;***************************************************************************
set_pagesize:
	lpm				; read table
	tst	r0			; check for end of table
	breq	spa_end		; no Pagesize to set
	cp	device,r0
	breq	spa_set		; C=0
	adiw	ZL,2			; skip MSB
	rjmp	set_pagesize	
spa_set:
	adiw	ZL,1			; Point to high Byte of Word
	lpm				; get Pagesize to R0
	mov Pagewords,r0
	mov Bcnt3,Pagewords	; initiate Counter
spa_end:
	ret
;***************************************************************************
;*
;* FUNCTION
;*	set_pollcode
;*
;* DESCRIPTION
;*	sets Code for Polling Flash for selected Device.
;*
;***************************************************************************
set_pollcode:
	lpm				; read table
	tst	r0			; check for end of table
	breq	spo_end		; no Pollcode to set
	cp	device,r0
	breq	spo_set		; C=0
	adiw	ZL,2			; skip MSB
	rjmp	set_pollcode	
spo_set:
	adiw	ZL,1			; Point to high Byte of Word
	lpm				; get Pagesize to R0
	mov PollcodeF,r0		; Set Pollcode for Flash Rom
spo_end:
	ret
;***************************************************************************
;*
;* FUNCTION
;*	healthcheck
;*
;* DESCRIPTION
;*	 flash led 2 times.
;*
;***************************************************************************
healthcheck:
	ldi s_data,2			; counter Register
LEDloop:
	rcall	LED_Green			; green LED on

	ldi temp1,250			; for 250 ms
	rcall delay

	rcall	LED_Yellow			; yellow LED on

	ldi temp1,250			; for 250 ms
	rcall delay

	dec s_data				; dec counter
	brne LEDloop
	rcall	LED_OFF			; all LED's OFF
	ret
;***************************************************************************
init_ports:				; init ports
	ldi	temp2,(1<<LEDY) | (1<<LEDG)
	out	LED_PORT,temp2
	ldi	temp2,(1<<LEDY) | (1<<LEDG)
	out	LED_DDR,temp2

	ldi	temp2,(1<<RST) | (1<<MOSI) | (1<<SCK)
	out	SPI_PORT,temp2
	ret
;***************************************************************************
catch_ports:			; set direction on ports
	ldi	temp2,(1<<RST) | (1<<MOSI) | (1<<SCK)
	out	SPI_DDR,temp2
	ret
;***************************************************************************
release_ports:		; release ports
	clr	temp2
	out	SPI_DDR,temp2
	ret
;***************************************************************************
LED_Green:			; LED green
	cbi LED_PORT,LEDG
	sbi LED_PORT,LEDY	;if bi-colour LED is used
	ret
;***************************************************************************
LED_Yellow:			; LED yellow
	cbi LED_PORT,LEDY
	sbi LED_PORT,LEDG	;if bi-colour LED is used
	ret
;***************************************************************************
LED_off:
	sbi LED_PORT,LEDG
	sbi LED_PORT,LEDY
	ret
;***************************************************************************
pas_RESET:			; set RESET passive
	tst	device
	brmi	m2		; S89 device
	sbi	SPI_PORT,RST
	rjmp	m3
m2:
	cbi	SPI_PORT,RST
m3:		
	ret
;***************************************************************************
act_RESET:			; set RST active
	tst	device
	brmi	m4		; S89 device
	cbi	SPI_PORT,RST
	rjmp	m5
m4:
	sbi	SPI_PORT,RST
m5:		
	ret
;***************************************************************************
.include "strings.inc"		;tables & strings
.include "usart.inc"		;usart routines
.include "delay.inc"		;delay routine
.include "spi.inc"		;spi functions
;***************************************************************************
.exit