;***************************************************************************
;*
;* FUNCTION
;*	 spi123
;*
;* DESCRIPTION
;*	 Write bytes 1 to 3 on the SPI. Byte 1 must be loaded into s_data
;*       Byte 2 ist addrh, Byte 3 ist addrl
;*
;***************************************************************************
spi123:
	rcall	wrser			; wrser(s_data) SPI write (byte 1)
	mov	s_data,addrh
	rcall	wrser			; wrser(addrh)  SPI write (byte 2)
	mov	s_data,addrl	
	rcall	wrser			; wrser(addrl)  SPI write (byte 3)
	ret
;***************************************************************************
;*
;* FUNCTION
;*	 w1234 (for Code simplification,)
;*
;* DESCRIPTION
;*	 Write SPI bytes 1 to 4.
;            Byte 1 must be loaded into cmd1
;            Byte 2 must be loaded into cmd2
;            Byte 3 must be loaded into cmd3
;            Byte 4 must be loaded into u_data
;*
;***************************************************************************
w1234:
	mov s_data,cmd1
	rcall wrser
	mov s_data,cmd2
	rcall wrser
	mov s_data,cmd3
	rcall wrser
	mov s_data,u_data		; get Byte 4 from serial
	rcall wrser
	ret
;***************************************************************************
;*
;* FUNCTION
;*	 w123r4 (for Code simplification)
;*
;* DESCRIPTION
;*	 Write SPI bytes 1 to 3, read Byte 4 to serial.
;            Byte 1 must be loaded into cmd1
;            Byte 2 must be loaded into cmd2
;            Byte 3 must be loaded into cmd3
;*
;***************************************************************************
w123r4:
	mov s_data,cmd1
	rcall wrser
	mov s_data,cmd2
	rcall wrser
	mov s_data,cmd3
	rcall wrser
	rcall rdser
	mov u_data,s_data		; put Byte 4 to serial
	rcall putc
	ret
;***************************************************************************
;*
;* FUNCTION
;*	rdser, wrser
;*
;* DESCRIPTION
;*	 Write and read bytes on the SPI.
;*
;***************************************************************************
rdser:	
	clr	s_data
wrser:	
	ldi	temp1,8		; load bit counter
	ldi	rd_s_data,0
wrs0:
	rol	s_data
	brcc	wrs1
	sbi	SPI_PORT,MOSI		; MOSI = 1
	rjmp	wrs2
wrs1:
	cbi	SPI_PORT,MOSI		; MOSI = 0
wrs2:
	lsl	rd_s_data
	sbic	SPI_PIN,MISO		; read MISO
	ori	rd_s_data,1

	rcall	Pulse_SCK

	dec	temp1			; advance bit counter
	brne	wrs0			; loop
	mov	s_data,rd_s_data
	ret
;***************************************************************************
Pulse_SCK:
	sbi	SPI_PORT,SCK
	ldi	temp2,(XTAL/1500+1)	
m0:	dec	temp2
	brne	m0

	cbi	SPI_PORT,SCK
	ldi	temp2,(XTAL/3000+1)	
m1:	dec	temp2
	brne	m1
	ret
;***************************************************************************
;*
;* FUNCTION
;*	read_send_progmem
;*
;* DESCRIPTION
;*	 Read one adress (2 Byte) from Program Memory and send it through UART
;*
;***************************************************************************
read_send_progmem:
	ldi	s_data,0x20		; read low Byte
	rcall	wrser			; wrser(0x28)  SPI write (byte 1)

	mov	s_data,addrh
	rcall	wrser			; wrser(addrh) SPI write (byte 2)

	mov	s_data,addrl	
	rcall	wrser			; wrser(addrl) SPI write (byte 3)

	rcall	rdser			; SPI read (byte 4)
	mov	u_data,s_data
	rcall	putc			; send data

	tst	device
	brmi	rsp3			; S89 device

	ldi	s_data,0x28		; read High Byte
	rcall	wrser			; wrser(0x20)  SPI write (byte 1)

	mov	s_data,addrh	
	rcall	wrser			; wrser(addrh) SPI write (byte 2)

	mov	s_data,addrl	
	rcall	wrser			; wrser(addrl) SPI write (byte 3)

	rcall	rdser			; SPI read  (byte 4)
	mov	u_data,s_data
	rcall	putc			; send data
rsp3:
	adiw	addrl,1		; Auto increment address
	ret
;***************************************************************************
;*
;* FUNCTION
;*	read_send_datamem
;*
;* DESCRIPTION
;*	 Read one Byte from Data Memory (eeprom) and send it through UART
;*
;***************************************************************************
read_send_datamem:		; Subroutine to read one eeprom Address
	tst	device		; no Support for S8252
	brmi	rsd3			; 

	ldi	s_data,0xa0
	rcall	wrser			; wrser(0xa0)   SPI write (byte 1)

	mov	s_data,addrh
	rcall	wrser			; wrser(addrh)  SPI write (byte 2)

	mov	s_data,addrl	
	rcall	wrser			; wrser(addrl)  SPI write (byte 3)

	rcall	rdser			; SPI read  (byte 4)
	mov	u_data,s_data

	rcall	putc			; send data

	adiw	addrl,1		; Auto increment address
	ret
rsd3:
	pop	temp1			; remove return Address from Stack in case of Error
	pop	temp1
	rjmp	put_err
;***************************************************************************
;*
;* FUNCTION
;*	eeprom_write
;*
;* DESCRIPTION
;*	 Write u_data to Data Memory (eeprom)
;*
;***************************************************************************
eeprom_write:
	tst	device
	brmi	eew3

	ldi	s_data,0xc0
	rcall	wrser			; wrser(0xc0)   SPI write (byte 1)

	mov	s_data,addrh
	rcall	wrser			; wrser(addrh)  SPI write (byte 2)

	mov	s_data,addrl
	rcall	wrser			; wrser(addrl)  SPI write (byte 3)

	mov	s_data,u_data
	rcall	wrser			; wrser(u_data) SPI write (byte 4)

	ldi	temp1,10		; delay 10mS
	rcall	delay

	adiw	addrl,1		; Auto increment address
	ret
eew3:
	pop temp1			; remove return Address from Stack in case of failure
	pop temp1
	rjmp	put_err
;***************************************************************************